#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/time.h>

#include "argument.h"
#include "text.h"
#include "correct.h"
#include "lib.h"

static int bufsize = PKT_BUFFER_SIZE_DEFAULT;

static Argument args[] = {
  { "-b", ARGUMENT_TYPE_INTEGER, &bufsize },
  { NULL, ARGUMENT_TYPE_NONE   , NULL     },
};

static int terminated = 0;
static void sigint_handler(int value)
{
  terminated = 1;
}

int main(int argc, char *argv[])
{
  int size;
  char *buffer;
  struct timeval tm;

  argument_read(&argc, argv, args);

  buffer = malloc(bufsize);
  if (buffer == NULL)
    error_exit("Out of memory.\n");

  while (!terminated) {
    size = pkt_text_read(stdin, buffer, bufsize, NULL, NULL, &tm);
    if (size < 0)
      break;
    if (size == bufsize)
      error_exit("Out of buffer.\n");

    pkt_correct_ethernet(buffer, size);

    signal(SIGINT , sigint_handler);
    signal(SIGTERM, sigint_handler);
    pkt_text_write(stdout, buffer, size, size, &tm);
    signal(SIGINT , SIG_DFL);
    signal(SIGTERM, SIG_DFL);
  }

  fflush(stdout);
  free(buffer);

  return 0;
}
